//
//  CLAppDelegate.m
//  NKDemo
//
//  Created by Darcy Liu on 8/24/12.
//  Copyright (c) 2012 Close To U. All rights reserved.
//

#import "CLAppDelegate.h"
#import <NewsstandKit/NewsstandKit.h>
#import "CLMainViewController.h"

@implementation CLAppDelegate

@synthesize window = _window;
@synthesize mainViewController;

- (void)dealloc
{
    [_window release];
    [super dealloc];
}

- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions
{
    self.window = [[[UIWindow alloc] initWithFrame:[[UIScreen mainScreen] bounds]] autorelease];
    // Override point for customization after application launch.
    
    [[NSUserDefaults standardUserDefaults] setBool:YES forKey:@"NKDontThrottleNewsstandContentNotifications"];
    
    [[UIApplication sharedApplication] registerForRemoteNotificationTypes:(UIRemoteNotificationTypeNewsstandContentAvailability | 
                                                                           UIRemoteNotificationTypeBadge |
                                                                           UIRemoteNotificationTypeSound |
                                                                           UIRemoteNotificationTypeAlert)];

    //[[UIApplication sharedApplication] setNewsstandIconImage:newIcon];
    [[UIApplication sharedApplication] setApplicationIconBadgeNumber:0];
    
    for (NKAssetDownload *asset in [[NKLibrary sharedLibrary] downloadingAssets]) {
        [asset downloadWithDelegate:self];
    }
    
    self.mainViewController = [[CLMainViewController alloc] initWithStyle:UITableViewStylePlain];
    self.window.rootViewController = [[UINavigationController alloc] initWithRootViewController:self.mainViewController];
    
    self.window.backgroundColor = [UIColor whiteColor];
    [self.window makeKeyAndVisible];
    return YES;
}

-(void)connection:(NSURLConnection *)connection didWriteData:(long long)bytesWritten totalBytesWritten:(long long)totalBytesWritten expectedTotalBytes:(long long)expectedTotalBytes{
    //NSLog(@"didWriteData %f (%lld /%lld)",1.f*totalBytesWritten/expectedTotalBytes,totalBytesWritten,expectedTotalBytes);

    [self.mainViewController updateProgressOfConnection:connection 
                                  withTotalBytesWritten:totalBytesWritten 
                                     expectedTotalBytes:expectedTotalBytes];
}

-(void)connectionDidResumeDownloading:(NSURLConnection *)connection totalBytesWritten:(long long)totalBytesWritten expectedTotalBytes:(long long)expectedTotalBytes {
    //NSLog(@"Resume downloading %f (%lld /%lld)",1.f*totalBytesWritten/expectedTotalBytes,totalBytesWritten,expectedTotalBytes);
    [self.mainViewController updateProgressOfConnection:connection 
                                  withTotalBytesWritten:totalBytesWritten 
                                     expectedTotalBytes:expectedTotalBytes];
}

-(void)connectionDidFinishDownloading:(NSURLConnection *)connection destinationURL:(NSURL *)destinationURL {

    
    NKAssetDownload *asset = [connection newsstandAssetDownload];
    NSString *fileName = [[asset userInfo] objectForKey:@"filename"];
    NSURL *fileURL = [[[asset issue] contentURL] URLByAppendingPathComponent:fileName];
    
    NSError *error = nil;
    [[NSFileManager defaultManager] moveItemAtURL:destinationURL toURL:fileURL error:&error];

    UIApplication *app = [UIApplication sharedApplication];
    [app setApplicationIconBadgeNumber:[app applicationIconBadgeNumber]+1];
    //UIImage *newIcon = [self getIconFromIssue:latestIssue]; // up to you
    //[app setNewsstandIconImage:newIcon];
    
    NSLog(@"FinishDownloading %@",error);
    [self.mainViewController updateIssue:connection];
    NSLog(@"move %@ %@",destinationURL,fileURL);
}

- (void)application:(UIApplication *)application didRegisterForRemoteNotificationsWithDeviceToken:(NSData *)newDeviceToken
{
    NSLog(@"newDeviceToken: %@",newDeviceToken);
}

-(void)application:(UIApplication *)application didFailToRegisterForRemoteNotificationsWithError:(NSError *)error
{
    NSLog(@"Failing in APNS registration: %@",error);
}

- (void)application:(UIApplication *)application didReceiveRemoteNotification:(NSDictionary *)userInfo {
    NSLog(@"Received a remote notification: %@", userInfo);
}

- (void)applicationWillResignActive:(UIApplication *)application
{
    // Sent when the application is about to move from active to inactive state. This can occur for certain types of temporary interruptions (such as an incoming phone call or SMS message) or when the user quits the application and it begins the transition to the background state.
    // Use this method to pause ongoing tasks, disable timers, and throttle down OpenGL ES frame rates. Games should use this method to pause the game.
}

- (void)applicationDidEnterBackground:(UIApplication *)application
{
    // Use this method to release shared resources, save user data, invalidate timers, and store enough application state information to restore your application to its current state in case it is terminated later. 
    // If your application supports background execution, this method is called instead of applicationWillTerminate: when the user quits.
}

- (void)applicationWillEnterForeground:(UIApplication *)application
{
    // Called as part of the transition from the background to the inactive state; here you can undo many of the changes made on entering the background.
}

- (void)applicationDidBecomeActive:(UIApplication *)application
{
    // Restart any tasks that were paused (or not yet started) while the application was inactive. If the application was previously in the background, optionally refresh the user interface.
}

- (void)applicationWillTerminate:(UIApplication *)application
{
    // Called when the application is about to terminate. Save data if appropriate. See also applicationDidEnterBackground:.
}

@end
