//
//  CLMainViewController.m
//  NKDemo
//
//  Created by Darcy Liu on 8/25/12.
//  Copyright (c) 2012 Close To U. All rights reserved.
//

#import "CLMainViewController.h"
#import <NewsstandKit/NewsstandKit.h>
#import "CLAppDelegate.h"
#import <QuartzCore/QuartzCore.h>

@interface CLMainViewController ()

@end

@implementation CLMainViewController


- (id)initWithStyle:(UITableViewStyle)style
{
    self = [super initWithStyle:style];
    if (self) {
        // Custom initialization
        server = [[CLServer alloc] init];
    }
    return self;
}

- (void)viewDidLoad
{
    [super viewDidLoad];

    // Uncomment the following line to preserve selection between presentations.
    // self.clearsSelectionOnViewWillAppear = NO;
 
    // Uncomment the following line to display an Edit button in the navigation bar for this view controller.
    //self.navigationItem.rightBarButtonItem = self.editButtonItem;
        
    UIActivityIndicatorView *loadingActivity = [[[UIActivityIndicatorView alloc] initWithActivityIndicatorStyle:UIActivityIndicatorViewStyleWhite] autorelease];
    [loadingActivity startAnimating];
    UIBarButtonItem *waitButton = [[UIBarButtonItem alloc] initWithCustomView:loadingActivity];
    [self.navigationItem setRightBarButtonItem:waitButton];
    
//    UIBarButtonItem *load = [[UIBarButtonItem alloc] initWithTitle:@"Download"
//                                                             style:UIBarButtonSystemItemDone
//                                                            target:self
//                                                            action:@selector(startDownload)];
//
//    self.navigationItem.leftBarButtonItem = load;
    
    
    [[NSNotificationCenter defaultCenter] addObserver:self 
                                             selector:@selector(IssuesDidUpdated:) 
                                                 name:IssuesDidUpdateNotification 
                                               object:server];
    
    [[NSNotificationCenter defaultCenter] addObserver:self 
                                             selector:@selector(IssuesFailedUpdated:) 
                                                 name:IssuesFailedUpdateNotification 
                                               object:server];
    [server getIssuesList];
}

-(void)IssuesDidUpdated:(NSNotification *)notification{
    UIBarButtonItem *refreshButton = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemRefresh target:self action:@selector(reloadIssues)];
    [self.navigationItem setRightBarButtonItem:refreshButton];
    [self.tableView reloadData];
}

-(void)IssuesFailedUpdated:(NSNotification *)notification{
    NSLog(@"%@",notification);
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Error"
                                                    message:@"Cannot get issues from server."
                                                   delegate:nil
                                          cancelButtonTitle:@"Close"
                                          otherButtonTitles:nil];
    [alert show];
    [alert release];
}

- (void)viewDidUnload
{
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    // Return the number of sections.
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return [server numberOfIssues];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString *CellIdentifier = @"CellIdentifier";
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    
    if (cell == nil) {
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:CellIdentifier];
        
        UIProgressView *processView = [[UIProgressView alloc] initWithProgressViewStyle:UIProgressViewStyleDefault];
        [processView setTag: 100];
        [processView setFrame: CGRectMake(54, 30, 220, processView.frame.size.height)];
        [processView setProgress:0.0f];
        [processView setAlpha:0.0f];
        [cell addSubview:processView];
        
        cell.imageView.image = [UIImage imageNamed:@"Icon.png"];
        //cell.imageView.layer.cornerRadius = 5;
        //cell.imageView.layer.masksToBounds = YES;
        cell.accessoryType = UITableViewCellAccessoryDetailDisclosureButton;
    }
    
    NKLibrary *library = [NKLibrary sharedLibrary];
    NKIssue *issue = [library issueWithName:[server nameOfIssueAtIndex:indexPath.row]];
    
    NSLog(@"%@ %d",[issue name],[issue status]);
    //NKIssueContentStatusNone,
    //NKIssueContentStatusDownloading,
    //NKIssueContentStatusAvailable,
    switch ([issue status]) {
        case NKIssueContentStatusAvailable:
            //NSLog(@"NKIssueContentStatusAvailable");
            cell.detailTextLabel.text = @"Available";
            break;
        case NKIssueContentStatusDownloading:
            //NSLog(@"NKIssueContentStatusDownloading");
            cell.detailTextLabel.text = @"Downloading....";
            break;
        case NKIssueContentStatusNone:
        default:
            //NSLog(@"NKIssueContentStatusNone");
            cell.detailTextLabel.text = @"Tap to download";
            break;
    }
    
    cell.textLabel.text = [[NSString alloc] initWithFormat:issue.name,indexPath.row];
    
    return cell;
}

#pragma mark - Table view delegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    [self.tableView deselectRowAtIndexPath:indexPath animated:YES];
    
    UITableViewCell *cell =  [self.tableView cellForRowAtIndexPath:indexPath];
    /*cell.detailTextLabel.alpha = 0.0f;
    
    UIProgressView *processView = (UIProgressView*)[cell viewWithTag:100];
    [processView setProgress: 0.5f];
    [processView setAlpha: 1.0f];*/
    NKLibrary *library = [NKLibrary sharedLibrary];
    NKIssue *issue = [library issueWithName:[server nameOfIssueAtIndex:indexPath.row]];
    if(issue.status==NKIssueContentStatusAvailable) {
        [self readIssue:issue];
    } else if(issue.status==NKIssueContentStatusNone) {
        [self downloadIssueAtIndex:indexPath.row];
        cell.detailTextLabel.text = @"Downloading....";
    }
    

}
#pragma mark - Issue actions

-(void)readIssue:(NKIssue *)issue {
    [[NKLibrary sharedLibrary] setCurrentlyReadingIssue:issue];
    QLPreviewController *previewController = [[[QLPreviewController alloc] init] autorelease];
    previewController.delegate = self;
    previewController.dataSource = self;
    [self presentModalViewController:previewController animated:YES];
}

-(void)downloadIssueAtIndex:(NSInteger)index {
    NSLog(@"%d",index);
    NKLibrary *library = [NKLibrary sharedLibrary];
    NKIssue *issue = [library issueWithName:[server nameOfIssueAtIndex:index]];   
    NSURL *downloadURL = [server contentURLForIssueWithName:issue.name];

    NSURLRequest *request = [NSURLRequest requestWithURL:downloadURL];
    NKAssetDownload *assetDownload = [issue addAssetWithRequest:request];
    
    NSDictionary *userInfo = [NSDictionary dictionaryWithObjectsAndKeys:
                                [NSNumber numberWithInt:index], @"Index",
                                @"preview.pdf", @"filename",nil];
    [assetDownload setUserInfo: userInfo];
    //[assetDownload setUserInfo:[NSDictionary dictionaryWithObject:[NSNumber numberWithInt:index] forKey:@"Index"]];
    //[assetDownload setUserInfo:[NSDictionary dictionaryWithObject:issue.name forKey:@"filename"]];
    [assetDownload downloadWithDelegate:(CLAppDelegate *)[[UIApplication sharedApplication] delegate]];
}

- (void)reloadIssues{
    NKLibrary *library = [NKLibrary sharedLibrary];
    NSLog(@"%@",library.issues);
    for (NKIssue *issue in [library issues]) {
        [library removeIssue:issue];
    }

    [server getIssuesList];
}

- (void)startDownload
{
//    NSLog(@"start Download");
//    NKLibrary *library = [NKLibrary sharedLibrary];
//    
//    NSString *issueName =@"test";
//    NSURLRequest *request = [NSURLRequest requestWithURL: [[NSURL alloc] initWithString:@"http://ioio.name/demo/newsstand/json.pdf"]];
//    
//    if (![library issueWithName:issueName]) {
//        NSLog(@"not esisted");
//        NKIssue *issue = [library addIssueWithName:issueName date:[[NSDate alloc] init]];
//        NKAssetDownload *asset = [issue addAssetWithRequest:request];
//        [asset setUserInfo:[NSDictionary dictionaryWithObject:@"contentFIlename" forKey:@"filename"]];
//        [asset downloadWithDelegate:(CLAppDelegate *)[[UIApplication sharedApplication] delegate]];
//    }
}

- (void)updateIssue:(NSURLConnection *)connection
{
    NKAssetDownload *assetDownload = connection.newsstandAssetDownload;
    NSIndexPath *indexPath = [NSIndexPath indexPathForRow:[[assetDownload.userInfo objectForKey:@"Index"] intValue] 
                                                inSection:0];
    NSLog(@"updateIssue: %d ",indexPath.row);
    UITableViewCell *cell = [self.tableView cellForRowAtIndexPath:indexPath];
    [cell.detailTextLabel setAlpha:1.0f];
    UIProgressView *progressView = (UIProgressView *)[cell viewWithTag:100];
    [progressView setAlpha:0.0f];
    [self.tableView reloadRowsAtIndexPaths:[NSArray arrayWithObjects:indexPath, nil] 
                          withRowAnimation:UITableViewRowAnimationFade];
}

- (void)updateProgressOfConnection:(NSURLConnection *)connection withTotalBytesWritten:(long long)totalBytesWritten expectedTotalBytes:(long long)expectedTotalBytes {
    // get asset
    NKAssetDownload *assetDownload = connection.newsstandAssetDownload;
    NSIndexPath *indexPath = [NSIndexPath indexPathForRow:[[assetDownload.userInfo objectForKey:@"Index"] intValue] inSection:0];
    UITableViewCell *cell = [self.tableView cellForRowAtIndexPath:indexPath];
    [cell.detailTextLabel setAlpha:0.0f];
    UIProgressView *progressView = (UIProgressView *)[cell viewWithTag:100];
    [progressView setAlpha:1.0f];
    [progressView setProgress:1.f*totalBytesWritten/expectedTotalBytes];
}

#pragma mark - QuickLook

- (NSInteger) numberOfPreviewItemsInPreviewController: (QLPreviewController *) controller {
    return 1;
}

- (id <QLPreviewItem>) previewController: (QLPreviewController *) controller previewItemAtIndex: (NSInteger) index {
    NKIssue *issue = [[NKLibrary sharedLibrary] currentlyReadingIssue];
    
    NSURL *issueURL = [NSURL fileURLWithPath:[server downloadPathForIssue:issue]];
    NSLog(@"Issue URL: %@",issueURL);
    return issueURL;
}

@end
